/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.WorkbenchException;

import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;
import de.bsvrz.buv.plugin.anlagenstatus.views.AnlagenStatus;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Aktion &Ouml;ffnen der Ansicht 'Anlagenstatus'.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenStatusViewOeffnenAktion.java 24448 2010-07-01 17:23:54Z gieseler $
 */
public class AnlagenStatusViewOeffnenAktion extends Action implements
		IWorkbenchWindowActionDelegate {

	/** Debug-Logger. */
	private final Debug log = Debug.getLogger();

	/**
	 * Konstruktor.
	 */
	public AnlagenStatusViewOeffnenAktion() {
		super("Ansicht 'Anlagenstatus' ffnen");
		setToolTipText("ffnet die Ansicht 'Anlagenstatus'");
		setId(AnlagenStatusViewOeffnenAktion.class.getName());
		setImageDescriptor(PluginAnlagenStatus.getDefault().getImageDescriptor(
				"icons/anlagenstatus.png"));
	}

	/** {@inheritDoc} */
	@Override
	public void run() {
		runWithEvent(null);
	}

	/**
	 * {@inheritDoc}.
	 * 
	 * @see org.eclipse.jface.action.Action#runWithEvent(org.eclipse.swt.widgets.Event)
	 */
	@Override
	public void runWithEvent(final Event event) {
		if (isEnabled()) {
			final IWorkbenchWindow dw = PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow();
			final IWorkbenchPage page = dw.getActivePage();

			try {
				page.showView(AnlagenStatus.VIEW_ID);
			} catch (final WorkbenchException e) {
				log.error(e.getLocalizedMessage());
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public boolean isEnabled() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		// TODO Auto-generated method stub

	}

	/** {@inheritDoc} */
	@Override
	public void init(final IWorkbenchWindow window) {
		// TODO Auto-generated method stub

	}

	/** {@inheritDoc} */
	@Override
	public void run(final IAction action) {
		runWithEvent(null);
	}

	/** {@inheritDoc} */
	@Override
	public void selectionChanged(final IAction action,
			final ISelection selection) {
		// TODO Auto-generated method stub

	}

}
