/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.actions;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.widgets.Event;

import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusEditPart;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;

/**
 * Aktion Anzeigen von Versorgungsinformationen.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: VersorgungAnzeigenAktion.java 26357 2010-09-17 08:57:49Z gieseler $
 */
public class VersorgungAnzeigenAktion extends Action {

	/** Der selektierte EditPart. */
	private final AnlagenStatusEditPart part;

	/** Attributgruppe. */
	private AttributeGroup atg;

	/** Aspekt. */
	private Aspect asp;

	/**
	 * Konstruktor.
	 * 
	 * @param part
	 *            EditPart
	 */
	public VersorgungAnzeigenAktion(final AnlagenStatusEditPart part) {
		super("Versorgungsdaten anzeigen");
		setToolTipText("Zeigt Versorgungsinformationen des Anlagenteils an");
		this.part = part;
	}

	/**
	 * Konstruktor zur Anzeige er Versorgungsdaten zu einer Attributgruppe und
	 * einem Aspekt.
	 * 
	 * @param part
	 *            EditPart
	 * @param atg
	 *            Attributgruppe
	 * @param asp
	 *            Aspekt
	 */
	public VersorgungAnzeigenAktion(final AnlagenStatusEditPart part,
			final AttributeGroup atg, final Aspect asp) {
		final String text = atg.getName() + " (" + atg.getPid() + ")"
				+ ", Aspekt: " + asp.getName();
		setText(text);
		setToolTipText("Daten " + text);
		this.part = part;
		this.atg = atg;
		this.asp = asp;
	}

	/**
	 * Erzeugt einen Men&uuml;eintrag f&uuml;r die Anzeige der Versorgungsdaten
	 * zu einer Attributgruppe und einem Aspekt.
	 * 
	 * @param part
	 *            EditPart
	 * @param atg
	 *            Attributgruppe
	 * @param asp
	 *            Aspekt
	 * @return Liste von Men&uuml;eintr&auml;gen
	 */
	public static List<Object> erzeugeMenuAktionen(
			final AnlagenStatusEditPart part, final AttributeGroup atg,
			final Aspect asp) {
		final List<Object> mlist = new ArrayList<Object>();

		if (!part.isInVersorgung(atg, asp)) {
			mlist.add(new VersorgungAnzeigenAktion(part, atg, asp));
		}

		return mlist;
	}

	/**
	 * Erzeugt alle Men&uuml;eintr&auml;ge f&uuml;r die Anzeige der
	 * Versorgungsdaten zu einer Attributgruppe.
	 * 
	 * @param part
	 *            EditPart
	 * @param atg
	 *            Attributgruppe
	 * @return Liste von Men&uuml;eintr&auml;gen
	 */
	public static List<Object> erzeugeMenuAktionen(
			final AnlagenStatusEditPart part, final AttributeGroup atg) {
		final List<Object> mlist = new ArrayList<Object>();

		if (atg.isParameter()) {
			final Aspect aspect = DaVVerbindung.getInstanz().getDaVVerbindung()
					.getDataModel().getAspect("asp.parameterSoll");
			if (!part.isInVersorgung(atg, aspect)) {
				mlist.add(new VersorgungAnzeigenAktion(part, atg, aspect));
			}
		} else {
			if (atg.getAspects().size() == 1) {
				final Aspect aspect = atg.getAspects().iterator().next();
				if (!part.isInVersorgung(atg, aspect)) {
					mlist.add(new VersorgungAnzeigenAktion(part, atg, atg
							.getAspects().iterator().next()));
				}
			} else {
				if (atg.getAspects().size() > 1) {
					final String text = atg.getName() + " (" + atg.getPid()
							+ ")";
					final IMenuManager manager = new MenuManager(text);
					for (Aspect asp : atg.getAspects()) {
						if (!part.isInVersorgung(atg, asp)) {
							manager.add(new VersorgungAnzeigenAktion(part, atg,
									asp));
						}
					}
					if (manager.getItems().length > 0) {
						mlist.add(manager);
					}
				}
			}
		}
		return mlist;
	}

	/**
	 * {@inheritDoc}.
	 * 
	 * @see org.eclipse.jface.action.Action#runWithEvent(org.eclipse.swt.widgets.Event)
	 */
	@Override
	public void runWithEvent(final Event event) {
		part.addVersorgungsDaten(atg, asp);
	}
}
