/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.einstellungen;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.draw2d.ColorConstants;

import de.bsvrz.buv.plugin.anlagenstatus.internal.AnlagenStatusKnotenStatus;

/**
 * Einstellungen f&uuml;r das Plug-in "Anlagenstatus".
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenStatusEinstellungen.java 26339 2010-09-16 14:19:06Z
 *          gieseler $
 */
public class AnlagenStatusEinstellungen {

	/**
	 * Hierarchierichtung Top/Down.
	 */
	private boolean topDown = PluginStandardEinstellungen.TOPDOWN_STANDARD;

	/**
	 * Hintergrundfarbe der Knoten.
	 */
	private long knotenBackGroundColor = ColorUtilities
			.rgbToLong(ColorConstants.gray.getRGB());

	/**
	 * Rahmenbreite der Knoten.
	 */
	private int rahmenBreite = 5;

	/**
	 * Knotenstatus Farben-Map.
	 */
	private Map<AnlagenStatusKnotenStatus, Long> statusFarben = new HashMap<AnlagenStatusKnotenStatus, Long>();

	/**
	 * FG Farben-Map.
	 */
	private final Map<AnlagenStatusFunktionsgruppe, Long> fgFarben = new HashMap<AnlagenStatusFunktionsgruppe, Long>();

	/**
	 * Steuermodule initial ausblenden ja/nein.
	 */
	private boolean steuerModuleAusblenden = PluginStandardEinstellungen.STEUERMODULE_AUSBLENDEN_STANDARD;

	/**
	 * Linienbreite der Verbindungslinien.
	 */
	private int linienBreite;

	/**
	 * Farbe der Verbindungslinien.
	 */
	private long linienFarbe;

	/**
	 * Konstruktor.
	 */
	public AnlagenStatusEinstellungen() {
		initStandardWerte();
	}

	/**
	 * Gibt die Map der FG-Farben zur&uuml;ck.
	 * 
	 * @return Map der FG-Farben
	 */
	public Map<AnlagenStatusFunktionsgruppe, Long> getFgFarben() {
		return fgFarben;
	}

	/**
	 * Setzt die Map der Status-Farben.
	 * 
	 * @param farben
	 *            Map der Status-Farben.
	 */
	public void setStatusFarben(
			final Map<AnlagenStatusKnotenStatus, Long> farben) {
		this.statusFarben = farben;
	}

	/**
	 * Gibt die Map der Status-Farben zur&uuml;ck.
	 * 
	 * @return Map der Status-Farben
	 */
	public Map<AnlagenStatusKnotenStatus, Long> getStatusFarben() {
		return statusFarben;
	}

	/**
	 * Gibt das Flag zur&uuml;ck, ob die Steuermodule initial ausgeblendet
	 * werden sollen.
	 * 
	 * @return true, wenn die Steuermodule initial ausgeblendet werden sollen,
	 *         sonst false
	 */
	public boolean isSteuerModuleAusblenden() {
		return steuerModuleAusblenden;
	}

	/**
	 * Setzt das Flag, ob die Steuermodule initial ausgeblendet werden sollen.
	 * 
	 * @param steuerModuleAusblenden
	 *            teuermodule initial ausgeblenden ja oder nein
	 */
	public void setSteuerModuleAusblenden(final boolean steuerModuleAusblenden) {
		this.steuerModuleAusblenden = steuerModuleAusblenden;
	}

	/**
	 * Gibt die Rahmenbreite der EditParts zur&uuml;ck.
	 * 
	 * @return Rahmenbreite
	 */
	public int getRahmenBreite() {
		return rahmenBreite;
	}

	/**
	 * Setzt die Rahkmenbreite der EditParts.
	 * 
	 * @param rahmenBreite
	 *            Rahmenbreite
	 */
	public void setRahmenBreite(final int rahmenBreite) {
		this.rahmenBreite = rahmenBreite;
	}

	/**
	 * Gibt die Hintergrundfarbe der Knoten zur&uuml;ck.
	 * 
	 * @return Hintergrundfarbe
	 */
	public long getKnotenBackGroundColor() {
		return knotenBackGroundColor;
	}

	/**
	 * Setzt die Hintergrundfarbe der Knoten.
	 * 
	 * @param knotenBackGroundColor
	 *            Hintergrundfarbe
	 * 
	 */
	public void setKnotenBackGroundColor(final long knotenBackGroundColor) {
		this.knotenBackGroundColor = knotenBackGroundColor;
	}

	/**
	 * Gibt das Flag zur&uuml;ck, ob die Hierarchieausrichtung Top/Down ist oder
	 * nicht.
	 * 
	 * @return true, wenn Top/Down sonst false
	 */
	public boolean isTopDown() {
		return topDown;
	}

	/**
	 * Setzt die Hierarchieausrichtung.
	 * 
	 * @param topDown
	 *            Hierarchieausrichtung Top/Down ja oder nein
	 */
	public void setTopDown(final boolean topDown) {
		this.topDown = topDown;
	}

	/**
	 * Gibt die Einstellungs-ID f&uuml;r diese KLasse zur&uuml;ck.
	 * 
	 * @return ID
	 */
	public static String getEinstellungenID() {
		return AnlagenStatusEinstellungen.class.getName();
	}

	/**
	 * Initialisiert die Standarddarstellung.
	 */
	public void initStandardWerte() {
		topDown = PluginStandardEinstellungen.TOPDOWN_STANDARD;
		knotenBackGroundColor = PluginStandardEinstellungen.KNOTEN_BACKGROUND_COLOR_STANDARD;
		rahmenBreite = PluginStandardEinstellungen.KNOTEN_RAHMENBREITE_STANDARD;
		linienBreite = PluginStandardEinstellungen.VERBINDUNGEN_LINIENBREITE_STANDARD;
		linienFarbe = PluginStandardEinstellungen.VERBINDUNGEN_LINIENFARBE_STANDARD;

		statusFarben.clear();
		statusFarben
				.put(AnlagenStatusKnotenStatus.UNBEKANNT,
						PluginStandardEinstellungen.KNOTEN_RAHMENFARBE_STATUS_UNBEKANNT);
		statusFarben
				.put(AnlagenStatusKnotenStatus.TEILGESTOERT,
						PluginStandardEinstellungen.KNOTEN_RAHMENFARBE_STATUS_TEILGESTOERT);
		statusFarben.put(AnlagenStatusKnotenStatus.GESTOERT,
				PluginStandardEinstellungen.KNOTEN_RAHMENFARBE_STATUS_GESTOERT);
		statusFarben.put(AnlagenStatusKnotenStatus.OK,
				PluginStandardEinstellungen.KNOTEN_RAHMENFARBE_STATUS_OK);

		fgFarben.clear();
		fgFarben.put(AnlagenStatusFunktionsgruppe.FG1,
				PluginStandardEinstellungen.FARBE_FG1);
		fgFarben.put(AnlagenStatusFunktionsgruppe.FG2,
				PluginStandardEinstellungen.FARBE_FG2);
		fgFarben.put(AnlagenStatusFunktionsgruppe.FG3,
				PluginStandardEinstellungen.FARBE_FG3);
		fgFarben.put(AnlagenStatusFunktionsgruppe.FG4,
				PluginStandardEinstellungen.FARBE_FG4);
		fgFarben.put(AnlagenStatusFunktionsgruppe.FG5,
				PluginStandardEinstellungen.FARBE_FG5);
		fgFarben.put(AnlagenStatusFunktionsgruppe.FG6,
				PluginStandardEinstellungen.FARBE_FG6);
		fgFarben.put(AnlagenStatusFunktionsgruppe.FG254,
				PluginStandardEinstellungen.FARBE_FG254);
		fgFarben.put(AnlagenStatusFunktionsgruppe.FG_UNBEKANNT,
				PluginStandardEinstellungen.KNOTEN_BACKGROUND_COLOR_STANDARD);

	}

	/**
	 * Gibt die Hintergrundfarbe f&uuml;r eine Funktionsgruppe zur&uuml:ck.
	 * 
	 * @param fg
	 *            AnlagenStatusFunktionsgruppe
	 * @return Farbe
	 */
	public Long getFGFarbe(final AnlagenStatusFunktionsgruppe fg) {
		return fgFarben.get(fg);
	}

	/**
	 * Gibt die Farbe zu einem KnotenStatus zur&uuml;ck.
	 * 
	 * @param status
	 *            AnlagenStatusKnotenStatus
	 * @return Farbe
	 */
	public long getKnotenStatusBorderColor(
			final AnlagenStatusKnotenStatus status) {
		return statusFarben.get(status);
	}

	/**
	 * Gibt die Linienbreite der Verbindungslinien zur&uuml:ck.
	 * 
	 * @return Linienbreite
	 */
	public int getLinienBreite() {
		return linienBreite;
	}

	/**
	 * Setzt die Linienbreite der Verbindungslinien.
	 * 
	 * @param linienBreite
	 *            Breite
	 */
	public void setLinienBreite(final int linienBreite) {
		this.linienBreite = linienBreite;
	}

	/**
	 * Gibt die Farbe der Verbindungslinien zur&uuml:ck.
	 * 
	 * @return Farbe
	 */
	public long getLinienFarbe() {
		return linienFarbe;
	}

	/**
	 * Setzt die Farbe der Verbindungslinien.
	 * 
	 * @param linienFarbe
	 *            Farbe
	 */
	public void setLinienFarbe(final long linienFarbe) {
		this.linienFarbe = linienFarbe;
	}

	/**
	 * Gibt die Farbe der Zusatzfenster zur&uuml:ck.
	 * 
	 * @return Farbe
	 */
	public long getZusatzFensterFarbe() {
		return PluginStandardEinstellungen.VERSORGUNG_FENSTER_BACKGROUNG_COLOR;
	}
}
