/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.einstellungen;

import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.DeAxl;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.DeLve;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.DeSys;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.DeUfd;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.DeVlt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.DeWww;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.DeWzg;

/**
 * Funktionsgruppendefinition.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */
public enum AnlagenStatusFunktionsgruppe {

	/** FG1. */
	FG1("FG1"),

	/** FG2. */
	FG2("FG2"),

	/** FG3. */
	FG3("FG3"),

	/** FG4. */
	FG4("FG4"),

	/** FG5. */
	FG5("FG5"),

	/** FG6. */
	FG6("FG6"),

	/** FG254. */
	FG254("FG254"),

	/** FG unbekannt. */
	FG_UNBEKANNT("unbekannte FG");

	/** Name der Funktionsgruppe. */
	private String name;

	/**
	 * Privater Konstruktor.
	 * 
	 * @param name
	 *            Name des Zustandes
	 */
	private AnlagenStatusFunktionsgruppe(final String name) {
		this.name = name;
	}

	/**
	 * Bestimmt die Funktionsgruppe eines Systemobjektes.
	 * 
	 * @param systemObjekt
	 *            SystemObjekt
	 * @return Funktionsgruppe
	 */
	public static final AnlagenStatusFunktionsgruppe bestimmeFG(
			final SystemObjekt systemObjekt) {
		if (systemObjekt instanceof DeLve) {
			return FG1;
		}

		if (systemObjekt instanceof DeAxl) {
			return FG2;
		}

		if (systemObjekt instanceof DeUfd) {
			return FG3;
		}

		if (systemObjekt instanceof DeWzg) {
			return FG4;
		}

		if (systemObjekt instanceof DeWww) {
			return FG5;
		}

		if (systemObjekt instanceof DeVlt) {
			return FG6;
		}

		if (systemObjekt instanceof DeSys) {
			return FG254;
		}

		return FG_UNBEKANNT;
	}

	/**
	 * Gibt den Namen der Funktionsgruppe zur&uuml;ck.
	 * 
	 * @return Name des Zustandes
	 */
	public String getName() {
		return name;
	}
}
