/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.einstellungen;

import org.eclipse.jface.action.IAction;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;
import de.bsvrz.buv.rw.bitctrl.eclipse.modell.java.JavaModellEinstellungen;
import de.bsvrz.buv.rw.bitctrl.eclipse.modell.java.JavaModellSaveAction;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Einstellungen des Plug-ins.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: PluginEinstellungen.java 25061 2010-07-25 22:55:24Z gieseler $
 */
public final class PluginEinstellungen {

	/** (einzige) Instanz der Klasse. */
	private static PluginEinstellungen instanz;

	/** Die Klasse mit den EInstellungen. */
	private AnlagenStatusEinstellungen einstellungen = new AnlagenStatusEinstellungen();

	/** Speichern mit Warten. */
	private final boolean speichernMitWarten = true;

	/**
	 * Gibt die Einstellungen zur&uuml;ck.
	 * 
	 * @return AnlagenStatusEinstellungen
	 */
	public AnlagenStatusEinstellungen getEinstellungen() {
		return einstellungen;
	}

	/**
	 * Privater Konstruktor.
	 */
	private PluginEinstellungen() {
	}

	/**
	 * Gibt die (einzige) Instanz der Klasse zur&uuml;ck.
	 * 
	 * @return Instanz
	 */
	public static PluginEinstellungen getInstance() {
		if (instanz == null) {
			instanz = new PluginEinstellungen();
			instanz.loadEinstellungen();
		}

		return instanz;
	}

	/**
	 * F&uuml;hrt die Speicheraktion aus.
	 * 
	 * @param aktion
	 *            Aktion
	 */
	private void speichernAktionAusfuehren(final IAction aktion) {
		PluginAnlagenStatus.getDefault().getLogger().info(
				"Einstellungen werden gespeichert");

		Debug.getLogger().info("Einstellungen werden gespeichert");

		if (speichernMitWarten) {
			final Cursor cursor = new Cursor(PlatformUI.getWorkbench()
					.getDisplay(), SWT.CURSOR_WAIT);

			final Shell activeShell = PlatformUI.getWorkbench().getDisplay()
					.getActiveShell();

			activeShell.setCursor(cursor);

			aktion.run();

			activeShell.setCursor(null);
			cursor.dispose();

			Debug.getLogger().info("Einstellungen wurden gespeichert");
		} else {
			PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
				public void run() {
					aktion.run();
				}
			});
		}
	}

	/**
	 * Speichert die Einstellungen.
	 */
	public void saveEinstellungen() {
		final IAction action = new JavaModellSaveAction<AnlagenStatusEinstellungen>(
				AnlagenStatusEinstellungen.class) {
			@Override
			protected String getCurrentName() {
				return AnlagenStatusEinstellungen.getEinstellungenID();
			}

			@Override
			protected AnlagenStatusEinstellungen getObject(final String newName) {
				return einstellungen;
			}
		};

		speichernAktionAusfuehren(action);
	}

	/**
	 * L&auml;dt die Einstellungen.
	 */
	private void loadEinstellungen() {
		final JavaModellEinstellungen<AnlagenStatusEinstellungen> e = new JavaModellEinstellungen<AnlagenStatusEinstellungen>(
				AnlagenStatusEinstellungen.class);
		AnlagenStatusEinstellungen b = null;
		if (PluginAnlagenStatus.getDefault().istOnline()) {
			b = e.getModellEinstellungen(EinstellungsArt.benutzerNetzweit,
					AnlagenStatusEinstellungen.getEinstellungenID());
		}
		if (b == null) {
			b = e.getModellEinstellungen(EinstellungsArt.benutzerLokal,
					AnlagenStatusEinstellungen.getEinstellungenID());
		}

		if (b != null) {
			einstellungen = b;
		}
	}
}
