/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.export;

/**
 * HTML.Tools.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * 
 * @version $Id: HtmlTools.java 32959 2012-02-29 15:05:11Z gieseler $
 */
public final class HtmlTools {

	/**
	 * Privater Konstruktor.
	 */
	private HtmlTools() {

	}

	/**
	 * Erzeugt eine HTML-Tabellenzeile.
	 * 
	 * @param attribute
	 *            Attribute der Zeile
	 * @param inhalt
	 *            Inhalt der Zeile
	 * @return String
	 */
	public static String erzeugeHtmlTabelleZeile(final String attribute,
			final String inhalt) {
		return erzeugeHtmlElement("tr", attribute, inhalt) + "\n";
	}

	/**
	 * Erzeugt eine HTML-Tabellenspalte.
	 * 
	 * @param attribute
	 *            Attribute der Spalte
	 * @param inhalt
	 *            Inhalt der Spalte
	 * @return String
	 */
	public static String erzeugeHtmlTabelleSpalte(final String attribute,
			final String inhalt) {
		return erzeugeHtmlElement("td", attribute, inhalt);
	}

	/**
	 * Erzeugt ein HTML-Element.
	 * 
	 * @param element
	 *            HTML-Element
	 * @param attribute
	 *            Attribute des Elements
	 * @param inhalt
	 *            Inhalt des Elements
	 * @return String
	 */
	public static String erzeugeHtmlElement(final String element,
			final String attribute, final String inhalt) {
		final StringBuffer sb = new StringBuffer();

		sb.append("<");
		sb.append(element);
		if (attribute != null) {
			sb.append(" " + attribute);
		}
		sb.append(">");

		if (inhalt != null) {
			sb.append(inhalt);
		}
		sb.append("</");
		sb.append(element);
		sb.append(">");

		return sb.toString();
	}
}
