/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.figures;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.draw2d.ActionEvent;
import org.eclipse.draw2d.ActionListener;
import org.eclipse.draw2d.Button;
import org.eclipse.draw2d.ButtonModel;
import org.eclipse.draw2d.ChangeEvent;
import org.eclipse.draw2d.ChangeListener;
import org.eclipse.draw2d.GridData;
import org.eclipse.draw2d.GridLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LabeledContainer;
import org.eclipse.draw2d.Panel;
import org.eclipse.draw2d.ScalableFigure;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.editparts.ZoomListener;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.ColorUtilities;
import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.PluginEinstellungen;
import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;
import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusEditPart;
import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusTopEditPart;
import de.bsvrz.buv.plugin.dobj.DoFigure;

/**
 * Figur (Fenster) zur Anzeige von Zusatzdaten.
 * <p>
 * Die Figur stellt einen Datenbereich und darunter einen Button zum
 * Schlie&szlig;en bereit.
 * </p>
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: DatenFenster.java 45198 2013-07-25 08:49:37Z gieseler $
 */
public abstract class DatenFenster extends LabeledContainer implements
		PropertyChangeListener, ScalableFigure, DoFigure, ZoomListener {

	/** Scale. */
	private final double scale = 1.0;

	/** Zoom. */
	private double zoom = 1.0;

	/**
	 * Parent Edit-Part.
	 */
	private final AnlagenStatusEditPart parentpart;

	/**
	 * 'Schliessen'-Button.
	 */
	private final Button closeButton;

	/**
	 * Button-Model.
	 */
	private final ButtonModel bModel;

	/**
	 * Datenbereich.
	 */
	private final Panel datenPanel;

	/**
	 * Konstruktor.
	 * 
	 * @param parentpart
	 *            Parent-Edit-Part.
	 */
	public DatenFenster(final AnlagenStatusEditPart parentpart) {
		super();
		this.parentpart = parentpart;
		setLayoutManager(new GridLayout(1, false));
		setFont(PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getWorkbench().getDisplay().getSystemFont());
		setBackgroundColor(ColorUtilities.longToColor(PluginEinstellungen
				.getInstance().getEinstellungen().getZusatzFensterFarbe()));

		// Datenbereich
		datenPanel = new Panel();
		datenPanel.setLayoutManager(new GridLayout(1, false));
		datenPanel.setFont(PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getWorkbench().getDisplay().getSystemFont());
		add(datenPanel);

		// Button zum Schliessen
		closeButton = new Button(PluginAnlagenStatus.getDefault()
				.getResourceBundle().getString("FensterSchliessen.button"));
		closeButton.setEnabled(true);
		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.HORIZONTAL_ALIGN_CENTER;
		gridData.verticalAlignment = GridData.VERTICAL_ALIGN_CENTER;
		getLayoutManager().setConstraint(closeButton, gridData);
		add(closeButton);

		bModel = new ButtonModel();
		// bModel.setSelected(true);
		bModel.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(final ActionEvent event) {
				// System.out.println("Action: " + event);
				// vpart.deactivate();
			}
		});

		bModel.addChangeListener(new ChangeListener() {
			public void handleStateChanged(final ChangeEvent event) {
				// System.out.println("VFIGURE StateChanged: "
				// + event.getPropertyName());

				if (event.getPropertyName().equals(
						ButtonModel.MOUSEOVER_PROPERTY)) {
					closeButton.getModel().setMouseOver(
							closeButton.getModel().isMouseOver());
				} else if (event.getPropertyName().equals(
						ButtonModel.ARMED_PROPERTY)) {
					closeButton.getModel().setArmed(
							closeButton.getModel().isArmed());
				} else if (event.getPropertyName().equals(
						ButtonModel.PRESSED_PROPERTY)) {
					parentpart.closeZusatzWindow(DatenFenster.this);
				}
			}
		});
		// closeButton.addMouseListener(this);

		setLocation(parentpart.getDatenFensterLocation());

		closeButton.setModel(bModel);
		getLayoutManager().layout(this);
		aktualisiereLayout();

		parentpart.getZoomManager().addZoomListener(this);

		final EditPart parent2 = parentpart.getParent();

		AbstractGraphicalEditPart tep = (AbstractGraphicalEditPart) parent2;

		final Rectangle bounds2 = getBounds();
		if (parentpart.isInView()) {
			while (!(tep instanceof AnlagenStatusTopEditPart)) {
				bounds2.x += tep.getFigure().getBounds().x;
				bounds2.y += tep.getFigure().getBounds().y;
				if (tep.getFigure().getBorder() != null) {
					final Insets insets = tep.getFigure().getBorder()
							.getInsets(this);
					bounds2.x += insets.left;
					bounds2.y += insets.top;
				}
				tep = (AbstractGraphicalEditPart) tep.getParent();
			}
			setBounds(bounds2);
			tep.getContentPane().add(this);
		} else {
			while ((tep instanceof AnlagenStatusEditPart)) {
				bounds2.x += tep.getFigure().getBounds().x;
				bounds2.y += tep.getFigure().getBounds().y;
				if (tep.getFigure().getBorder() != null) {
					final Insets insets = tep.getFigure().getBorder()
							.getInsets(this);
					bounds2.x += insets.left;
					bounds2.y += insets.top;
				}
				tep = (AbstractGraphicalEditPart) tep.getParent();
			}
			final RootEditPart root = parentpart.getRoot();
			final ScalableRootEditPart srep = (ScalableRootEditPart) root;

			// GUIDE_LAYER ist ganz oben und wird in Ansichten nicht benutzt
			// HANDLE-LAYER geht auch, aber dann werden die Selektionslinien
			// ber die Fenster gezeichnet
			final IFigure layer = srep.getLayer(LayerConstants.GUIDE_LAYER);
			layer.add(this);
		}
	}

	/**
	 * Aktualisiert das Layout.
	 */
	protected void aktualisiereLayout() {
		setSize(getPreferredSize());
		getLayoutManager().layout(this);
	}

	/**
	 * F&uuml;gt eine Figur zum Datenbereich hinzu.
	 * 
	 * @param figure
	 *            Figur
	 */
	protected void addDatenFigure(final IFigure figure) {
		datenPanel.add(figure);
		aktualisiereLayout();
	}

	/** {@inheritDoc} */
	@Override
	public void propertyChange(final PropertyChangeEvent evt) {
		final String prop = evt.getPropertyName();
		if (OnlineDatenFigure.DATEN_UPDATE_PROPERTY.equals(prop)) {
			aktualisiereLayout();
		}
	}

	/**
	 * Gibt den Parent Edit-Part zur&uuml;ck.
	 * 
	 * @return Parent Edit-Part
	 */
	public AnlagenStatusEditPart getParentpart() {
		return parentpart;
	}

	/** {@inheritDoc} */
	@Override
	public double getScale() {
		return scale;
	}

	/** {@inheritDoc} */
	@Override
	public void setScale(final double scale) {
		// if (this.scale == scale) {
		// return;
		// }
		//
		// this.scale = scale;
		// updateFigure();

		//
		// // Dieses Fenster liegt auf einem unskalierten Layer (GUIDE_LAYER,
		// s.o.)
		// // deshalb mssen hier die Eckpunkte von den (vom Editpart geerbten)
		// // skalierten Wert zurckgerechnet werden
		// final Rectangle bounds2 = getBounds();
		// bounds2.x *= scale;
		// bounds2.y *= scale;
		// setBounds(bounds2);
		// repaint();
	}

	/** {@inheritDoc} */
	@Override
	public void zoomChanged(final double newzoom) {
		handleZoomChanged(newzoom);

	}

	/** {@inheritDoc} */
	@Override
	public void handleZoomChanged(final double newZoom) {
		if (zoom == newZoom) {
			return;
		}

		zoom = newZoom;
		updateFigure();
	}

	/**
	 * Aktualisiert die Figure.
	 */
	private void updateFigure() {
		final double combinedScale = zoom / scale;

		if (!parentpart.isInView()) {
			setLocation(parentpart.getDatenFensterLocation());
		}

		// Dieses Fenster liegt auf einem unskalierten Layer (GUIDE_LAYER, s.o.)
		// deshalb mssen hier die Eckpunkte von den (vom Editpart geerbten)
		// skalierten Wert zurckgerechnet werden
		final Rectangle bounds2 = getBounds();
		bounds2.x *= combinedScale;
		bounds2.y *= combinedScale;
		setBounds(bounds2);
		aktualisiereLayout();
		repaint();
	}

	/** {@inheritDoc} */
	@Override
	public Point getHotspot() {
		return getLocation();
	}

	/** {@inheritDoc} */
	@Override
	public void setHotspot(final Point hotspot) {
		setLocation(hotspot);
	}

	/** {@inheritDoc} */
	@Override
	public void setSichtbareZoomStufe(final double vonZoom, final double bisZoom) {
	}
}
