/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.anlagenstatus.figures;

import org.eclipse.draw2d.GridLayout;
import org.eclipse.draw2d.Label;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusEditPart;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.NumberValue;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.DeLve;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.DeWzg;

/**
 * Figur zur Anzeige der Erg&auml;zenden DE-Fehler.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: DeErgaenzendeFehlermeldungFigure.java 32959 2012-02-29 15:05:11Z gieseler $
 */
public abstract class DeErgaenzendeFehlermeldungFigure extends
		OnlineDatenFigure {

	/** TLS-Fehlerbyte 1. */
	private Label deTLSFehlerByte1;

	/** TLS-Fehlerbyte 2. */
	private Label deTLSFehlerByte2;

	/** TLS-Fehlerbyte 3. */
	private Label deTLSFehlerByte3;

	/** TLS-Fehlerbyte 4. */
	private Label deTLSFehlerByte4;

	/** Hersteller. */
	private Label deTLSHersteller;

	/**
	 * Konstruktor.
	 * 
	 * @param parentpart
	 *            Parent-Part
	 */
	public DeErgaenzendeFehlermeldungFigure(
			final AnlagenStatusEditPart parentpart) {
		super(parentpart);
		setLayoutManager(new GridLayout(2, false));
		setFont(PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getWorkbench().getDisplay().getSystemFont());
	}

	/**
	 * Erzeugt die Tabelle der Daten.
	 */
	protected void erzeugeTabelle() {
		add(new Label("Hersteller:"));
		deTLSHersteller = new Label();
		add(deTLSHersteller);

		add(new Label("TLS-Fehlerbyte 1:"));
		deTLSFehlerByte1 = new Label();
		add(deTLSFehlerByte1);

		add(new Label("TLS-Fehlerbyte 2:"));
		deTLSFehlerByte2 = new Label();
		add(deTLSFehlerByte2);

		add(new Label("TLS-Fehlerbyte 3:"));
		deTLSFehlerByte3 = new Label();
		add(deTLSFehlerByte3);

		add(new Label("TLS-Fehlerbyte 4:"));
		deTLSFehlerByte4 = new Label();
		add(deTLSFehlerByte4);
	}

	/**
	 * Statische Methode zur Erzeugung der Figure.
	 * 
	 * @param parentpart
	 *            Parent-Part
	 * @return Figure
	 */
	public static DeErgaenzendeFehlermeldungFigure erzeugeFigure(
			final AnlagenStatusEditPart parentpart) {
		final SystemObjekt systemObjekt = parentpart.getModel()
				.getSystemObjekt();

		if (systemObjekt instanceof DeLve) {
			return new DeErgaenzendeFehlermeldungFigureFG1(parentpart);
		} else if (systemObjekt instanceof DeWzg) {
			return new DeErgaenzendeFehlermeldungFigureFG4(parentpart);
		}

		return null;
	}

	/** {@inheritDoc} */
	@Override
	public void showDaten(final ResultData result) {
		if (deTLSFehlerByte1 == null) {
			removeAll();
			erzeugeTabelle();
		}

		// final DataDescription desc = result.getDataDescription();
		final Data data = result.getData();

		deTLSHersteller.setText(data.getItem("Hersteller").asTextValue()
				.getText());

		final Data.NumberArray tlsFehler = data.getItem("FehlercodesTLS")
				.getUnscaledArray("Fehler");
		final NumberValue fehlerbyte1 = tlsFehler.getValue(0);
		final NumberValue fehlerbyte2 = tlsFehler.getValue(1);
		final NumberValue fehlerbyte3 = tlsFehler.getValue(2);
		final NumberValue fehlerbyte4 = tlsFehler.getValue(3);

		deTLSFehlerByte1.setText(fehlerbyte1.getText() + ", "
				+ convertByte1(fehlerbyte1.intValue()));
		deTLSFehlerByte2.setText(fehlerbyte2.getText() + ", "
				+ convertByte2(fehlerbyte2.intValue()));
		deTLSFehlerByte3.setText(fehlerbyte3.getText()
				+ convertByte3(fehlerbyte3.intValue()));
		deTLSFehlerByte4.setText(fehlerbyte4.getText()
				+ convertByte4(fehlerbyte4.intValue()));
	}

	/**
	 * Konvertiert das 1. TLS-Fehlerbyte.
	 * 
	 * @param tlsbyte
	 *            TLS-Fehlerbyte
	 * @return Text
	 */
	private String convertByte1(final int tlsbyte) {

		final StringBuilder result = new StringBuilder();

		// Byte 1
		for (int i = 0; i < 8; i++) {
			String addMsg = null;

			if ((tlsbyte & (1 << i)) != 0) {

				switch (i) {
				case 0:
					addMsg = "Teilstrung"; //$NON-NLS-1$
					break;
				case 4:
					addMsg = "Lokalbus ausgefallen"; //$NON-NLS-1$
					break;
				case 5:
					addMsg = "RNR am Lokalbus"; //$NON-NLS-1$
					break;
				default:
					break;
				}
			}

			if (addMsg != null) {
				if (result.length() > 0) {
					result.append(", ");
				}
				result.append(addMsg);
			}
		}
		if (result.toString().length() == 0) {
			return ("keine Fehler");
		}

		return result.toString();
	}

	/**
	 * Konvertiert das 2. TLS-Fehlerbyte.
	 * 
	 * @param tlsbyte
	 *            TLS-Fehlerbyte
	 * @return Text
	 */
	public String convertByte2(final int tlsbyte) {
		return "";
	}

	/**
	 * Konvertiert das 3. TLS-Fehlerbyte.
	 * 
	 * @param tlsbyte
	 *            TLS-Fehlerbyte
	 * @return Text
	 */
	public String convertByte3(final int tlsbyte) {
		return "";
	}

	/**
	 * Konvertiert das 4. TLS-Fehlerbyte.
	 * 
	 * @param tlsbyte
	 *            TLS-Fehlerbyte
	 * @return Text
	 */
	public String convertByte4(final int tlsbyte) {
		return "";
	}

}
