/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.figures;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Panel;
import org.eclipse.draw2d.ScalableFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;

import de.bsvrz.buv.plugin.dobj.DoFigure;
import de.bsvrz.buv.plugin.dobj.decorator.ZoomVerhaltenFixFigure;

/**
 * EAK-Figure.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: EakFigure.java 46802 2013-10-23 15:13:00Z gieseler $
 */
public class EakFigure extends Panel implements ScalableFigure, DoFigure,
		ZoomVerhaltenFixFigure {

	/** Scale. */
	private double scale = 1.0;

	/** Zoom. */
	private double zoom = 1.0;

	/** Dimension. */
	private Dimension dimension;

	/** Flag ZoomFix. */
	private boolean zoomFix;

	/**
	 * Konstruktor.
	 */
	public EakFigure() {
		super();
		setLayoutManager(new XYLayout());
	}

	/**
	 * Setzt die Gr&ouml;sse der Figur.
	 * 
	 * @param dimension
	 *            Gr&ouml;sse der Figur
	 */
	public void setDimension(final Dimension dimension) {
		this.dimension = dimension;
	}

	/** {@inheritDoc} */
	@Override
	public double getScale() {
		return scale;
	}

	/** {@inheritDoc} */
	@Override
	public void setScale(final double scale) {
		if (getParent() instanceof AnlagenGeraetFigure) {
			return;
		}

		if (this.scale == scale) {
			return;
		}

		this.scale = scale;

		updateFigure();
		repaint();
	}

	/**
	 * Aktualisiert die Figure.
	 */
	public void updateFigure() {
		if (dimension == null) {
			dimension = getSize();
		}

		final double combinedScale = zoom / scale;
		int width = dimension.width;
		int height = dimension.height;

		if (isZoomVerhaltenFix()) {
			width = (int) (dimension.width / combinedScale);
			if ((width % 2) != 0) {
				width++;
			}
			height = (int) (dimension.height / combinedScale);
			if ((height % 2) != 0) {
				height++;
			}
		} else {
			width = (int) (dimension.width * scale);
			if ((width % 2) != 0) {
				width++;
			}
			height = (int) (dimension.height * scale);
			if ((height % 2) != 0) {
				height++;
			}
		}

		setSize(width, height);
	}

	/** {@inheritDoc} */
	@Override
	protected final void paintChildren(final Graphics graphics) {
		final double amount;
		if (isZoomVerhaltenFix()) {
			amount = 1 / zoom / scale;
		} else {
			amount = scale;
		}

		if (amount != 1.0) {
			graphics.scale(amount);
			graphics.pushState();
			super.paintChildren(graphics);
			graphics.popState();
		} else {
			super.paintChildren(graphics);
		}
	}

	/** {@inheritDoc} */
	@Override
	protected final boolean useLocalCoordinates() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public void handleZoomChanged(final double newZoom) {
		if (getParent() instanceof AnlagenGeraetFigure) {
			return;
		}

		if (zoom == newZoom) {
			return;
		}

		zoom = newZoom;

		updateFigure();
	}

	/** {@inheritDoc} */
	@Override
	public Point getHotspot() {
		return getLocation();
	}

	/** {@inheritDoc} */
	@Override
	public void setHotspot(final Point hotspot) {
		setLocation(hotspot);
	}

	/** {@inheritDoc} */
	@Override
	public void setSichtbareZoomStufe(final double vonZoom, final double bisZoom) {
	}

	/** {@inheritDoc} */
	@Override
	public boolean isZoomVerhaltenFix() {
		return zoomFix;
	}

	/** {@inheritDoc} */
	@Override
	public void setZoomVerhaltenFix(boolean zoomVerhaltenFix) {
		zoomFix = zoomVerhaltenFix;
	}
}
