/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.internal;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;

import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusModell;
import de.bsvrz.buv.rw.basislib.berechtigung.DaVKommunikation;
import de.bsvrz.buv.rw.bitctrl.AbstractDAVPlugin;
import de.bsvrz.buv.rw.rw.Activator;
import de.bsvrz.sys.funclib.bitctrl.daf.DavProvider;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactoryServiceTracker;

/**
 * Der Eclipse-Framework-Aktivator fr das Plugin "Anlagenstatus".
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: PluginAnlagenStatus.java 26696 2010-10-13 14:18:56Z gieseler $
 */
public class PluginAnlagenStatus extends AbstractDAVPlugin implements
		PropertyChangeListener {

	/** Die ID des Plugins. */
	public static final String PLUGIN_ID = "de.bsvrz.buv.plugin.anlagenstatus";

	/** Die globale Instanz des Plugins. */
	private static PluginAnlagenStatus plugin;

	/** Das Ressource bundle. */
	private ResourceBundle resourceBundle;

	/** Tracker ObjektFactory. */
	private ObjektFactoryServiceTracker tracker;

	/** Lock-Objekt. */
	private final Object pluginLock = new Object();

	/** Modell. */
	private AnlagenStatusModell anlagenModell;

	/**
	 * The constructor.
	 */
	public PluginAnlagenStatus() {
		super();
		try {
			resourceBundle = ResourceBundle
					.getBundle("de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatusResources"); //$NON-NLS-1$
		} catch (final MissingResourceException x) {
			resourceBundle = null;
		}
	}

	/** {@inheritDoc} */
	@Override
	public void start(final BundleContext context) throws Exception {
		super.start(context);
		tracker = new ObjektFactoryServiceTracker(context);
		tracker.open();

		plugin = this;

		if (getObjektFactory() == null) {
			throw new BundleException(
					"Von der Applikation wird kein ObjektFactory-Service fr die Nutzerverbindung bereitgestellt. Erwartet wird ein Service der die Schnittstelle "
							+ ObjektFactory.class
							+ " implementiert und die Eigenschaft name=Nutzerverbindung besitzt.");
		}

		getObjektFactory().addPropertyChangeListener(this);

		if (getObjektFactory().isVerbunden()) {
			startOnline();
		} else {
			startOffline();
		}

	}

	/**
	 * Startet Online-Mode.
	 */
	private void startOnline() {
		getLogger().info("Starte Online-Mode");
		synchronized (pluginLock) {
			Activator
					.getDefault()
					.getLog()
					.log(new Status(IStatus.INFO,
							PluginAnlagenStatus.PLUGIN_ID,
							"Initialisiere Berechtigungsverwaltung Anlagenstatus"));
			DaVKommunikation.getInstanz()
					.hinzufuegenOberflaechenBenutzerZuhoerer(
							AnlagenStatusBerechtigungsVerwaltung.getInstanz());
		}
	}

	/**
	 * Startet Offline-Mode.
	 */
	private void startOffline() {
		getLogger().info("Starte Offline-Mode");
		synchronized (pluginLock) {
		}
	}

	/**
	 * Gibt eine Objektfabrik zurck, die als OSGi-Service zur Verfgung
	 * gestellt wurde.
	 * 
	 * @return eine Objektfabrik.
	 */
	@Override
	public ObjektFactory getObjektFactory() {
		return tracker.getObjektFactory();
	}

	/**
	 * Gibt das Modell zur&uuml;ck.
	 * 
	 * @return {@link AnlagenStatusModell}
	 */
	public AnlagenStatusModell getAnlagenModell() {
		if (anlagenModell == null) {
			anlagenModell = new AnlagenStatusModell();
		}

		return anlagenModell;
	}

	/**
	 * Gibt den Online-Modus zur&uuml;ck.
	 * 
	 * @return true wenn online, sonst false
	 */
	public boolean istOnline() {
		return getObjektFactory().isVerbunden();
	}

	/** {@inheritDoc} */
	@Override
	public void stop(final BundleContext context) throws Exception {
		tracker.close();
		plugin = null;
		super.stop(context);
	}

	/**
	 * liefert die globale Instanz.
	 * 
	 * @return die Instanz
	 */
	public static PluginAnlagenStatus getDefault() {
		return plugin;
	}

	/**
	 * liefert einen Imagedescriptor fr den bergebenen Dateipfad.
	 * 
	 * @param path
	 *            der Pfad.
	 * @return der Descriptor.
	 */
	@Override
	public ImageDescriptor getImageDescriptor(final String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin(
				PluginAnlagenStatus.PLUGIN_ID, path);
	}

	/**
	 * ermittelt des Bundle mit den Plugin-Ressourcen.
	 * 
	 * @return das Resource-Bundle.
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	/** {@inheritDoc} */
	@Override
	public void propertyChange(final PropertyChangeEvent evt) {
		if (DavProvider.PROP_VERBUNDEN.equals(evt.getPropertyName())) {
			if (Boolean.TRUE.equals(evt.getNewValue())) {
				getLogger().info("Anlagenstatus-Plugin: Verbindung aufgebaut");
				startOnline();
			} else {
				getLogger().info("Anlagenstatus-Plugin: Verbindung abgebaut");
				startOffline();
			}
			// setViewsOnline(Boolean.TRUE.equals(evt.getNewValue()));
		}

	}
}
