/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.modell;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.Bendpoint;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.AnschlussPunkt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.AnschlussPunktKommunikationsPartner;

/**
 * Verbindung zwischen Anlagenstatus-Knoten.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenStatusConnection.java 33710 2012-04-20 11:06:38Z gieseler $
 */
public class AnlagenStatusConnection implements
		Comparable<AnlagenStatusConnection> {

	private AnlagenStatusKnoten sourceKnoten;
	private AnlagenStatusKnoten m_target;

	public AnschlussPunkt getAnschlussPunkt() {
		return anschlussPunkt;
	}

	public AnschlussPunktKommunikationsPartner getAnschlussPunktKommunikationsPartner() {
		return anschlussPunktKommunikationsPartner;
	}

	private final AnschlussPunkt anschlussPunkt;
	private final AnschlussPunktKommunikationsPartner anschlussPunktKommunikationsPartner;

	private final Integer sourcePort;
	private final Integer targetPort;

	public int getSourcePort() {
		return sourcePort;
	}

	public int getTargetPort() {
		return targetPort;
	}

	public AnlagenStatusConnection(
			final AnlagenStatusGeraetKnoten srcKnoten,
			final AnschlussPunkt anschlussPunkt,
			final AnlagenStatusKnoten targetKnoten,
			final AnschlussPunktKommunikationsPartner anschlussPunktKommunikationsPartner) {
		this.anschlussPunkt = anschlussPunkt;
		this.anschlussPunktKommunikationsPartner = anschlussPunktKommunikationsPartner;
		sourcePort = anschlussPunkt.getKdAnschlussPunkt().getDatum()
				.getPortNummer().intValue();
		targetPort = anschlussPunktKommunikationsPartner
				.getKdAnschlussPunktKommunikationsPartner().getDatum()
				.getPortNummer().intValue();

		setSource(srcKnoten);
		setTarget(targetKnoten);
		buildName();
		setSichtbar(srcKnoten.isSichtbar() && targetKnoten.isSichtbar());
	}

	private void buildName() {
		setName("Verbindung" + System.getProperty("line.separator")
				+ getSource().getLabel() + ", Anschlupunkt: "
				+ anschlussPunkt.getName() + " ["
				+ getSource().getKnotenNummer() + "." + sourcePort + "]"
				+ System.getProperty("line.separator") + "--> "
				+ System.getProperty("line.separator") + getTarget().getLabel()
				+ ", Anschlupunktkommunikationspartner: "
				+ anschlussPunktKommunikationsPartner.getName() + "["
				+ getTarget().getKnotenNummer() + "." + targetPort + "]");
	}

	/** Sets the connection source. */
	public void setSource(final AnlagenStatusKnoten iSource) {
		sourceKnoten = iSource;
	}

	/** Sets the connection target. */
	public void setTarget(final AnlagenStatusKnoten iTarget) {
		m_target = iTarget;
	}

	/** Gets the connection source. */
	public AnlagenStatusKnoten getSource() {
		return sourceKnoten;
	}

	/** Gets the connection target. */
	public AnlagenStatusKnoten getTarget() {
		return m_target;
	}

	/**
	 * List of bendpoint model object associated with current connection model
	 * object.
	 */
	protected List bendpoints = new ArrayList();

	/** Returns the list of bendpoints model objects. */
	public List getBendpoints() {
		return bendpoints;
	}

	/** Property change support. */
	protected PropertyChangeSupport listeners = new PropertyChangeSupport(this);

	public void addPropertyChangeListener(final PropertyChangeListener l) {
		listeners.addPropertyChangeListener(l);
	}

	public void removePropertyChangeListener(final PropertyChangeListener l) {
		listeners.removePropertyChangeListener(l);
	}

	protected void firePropertyChange(final String prop, final Object old,
			final Object newValue) {
		listeners.firePropertyChange(prop, old, newValue);
	}

	/**
	 * Inserts a bendpoint.
	 **/
	public void insertBendpoint(final int index, final Bendpoint point) {
		getBendpoints().add(index, point);
		firePropertyChange("bendpoint", null, null);
	}

	/**
	 * Removes a bendpoint.
	 **/
	public void removeBendpoint(final int index) {
		getBendpoints().remove(index);
		firePropertyChange("bendpoint", null, null);
	}

	/**
	 * Sets another location for an existing bendpoint.
	 **/
	public void setBendpoint(final int index, final Bendpoint point) {
		getBendpoints().set(index, point);
		firePropertyChange("bendpoint", null, null);
	}

	/** Source terminal name. */
	protected String m_sourceTerminal;

	/** Target terminal name. */
	protected String m_targetTerminal;

	/**
	 * Sets the name of the source terminal.
	 **/
	public void setSourceTerminal(final String s) {
		m_sourceTerminal = s;
	}

	/**
	 * Sets the name of the target terminal.
	 **/
	public void setTargetTerminal(final String s) {
		m_targetTerminal = s;
	}

	/** Gets source terminal name. */
	public String getSourceTerminal() {
		return m_sourceTerminal;
	}

	/** Gets source terminal name. */
	public String getTargetTerminal() {
		return m_targetTerminal;
	}

	/** For debug purposes... */
	private String m_name = "Anonymous Connection";

	private boolean endeLabel = true;

	public void setEndeLabel(final boolean endeLabel) {
		this.endeLabel = endeLabel;
	}

	private boolean anfangLabel = true;
	private boolean sichtbar;

	public boolean isSichtbar() {
		return sichtbar;
	}

	public void setAnfangLabel(final boolean anfangLabel) {
		this.anfangLabel = anfangLabel;
	}

	public void setName(final String iName) {
		m_name = iName;
	}

	@Override
	public String toString() {
		return m_name;
	}

	/** {@inheritDoc} */
	@Override
	public int compareTo(final AnlagenStatusConnection arg0) {
		return sourcePort.compareTo(targetPort);
	}

	/**
	 * @return
	 */
	public Dimension getSourceLabelDimension() {
		return calcLabelDimension(sourcePort.toString());
	}

	public Dimension getTargetLabelDimension() {
		return calcLabelDimension(targetPort.toString());
	}

	private Dimension calcLabelDimension(final String text) {
		final Label l = new Label(text);
		l.setFont(PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getWorkbench().getDisplay().getSystemFont());
		final Dimension size = l.getTextBounds().getSize();

		// Versatz zur Linie
		size.width += 15;
		size.height += 15;
		return size;
	}

	public boolean istMitEndeLabel() {
		return endeLabel;
	}

	public boolean istMitAnfangLabel() {
		// TODO Auto-generated method stub
		return anfangLabel;
	}

	public void setSichtbar(final boolean b) {
		sichtbar = b;
	}

}
