/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.modell;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.plugin.anlagenstatus.internal.AnlagenStatusKnotenStatus;
import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusEditPart;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateEvent;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateListener;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatensatz;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.metamodellglobal.attribute.AttJaNein;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.attribute.AttTlsDEFehlerStatus;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.attribute.AttTlsDEKanalStatus;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.attribute.AttTlsSysKnotenNummer;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.konfigurationsdaten.KdDe.Daten;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.De;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.DeSys;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.onlinedaten.OdTlsGloDeFehler;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * DE im Anlagenstatus-Modell.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenStatusDeKnoten.java 43864 2013-05-14 11:10:42Z hoesel $
 */
public class AnlagenStatusDeKnoten extends AnlagenStatusKnoten implements
		DatensatzUpdateListener {

	/** die Liste der DAV-Anmeldungen. */
	private final List<OnlineDatensatz<?>> anmeldungen = new ArrayList<OnlineDatensatz<?>>();

	/**
	 * Konstruktor.
	 * 
	 * @param modell
	 *            Anlagenstatus-Modell
	 * @param parent
	 *            Parent
	 * @param systemObjekt
	 *            SystemObjekt
	 * @param level
	 *            Hierarchie-Level
	 */
	public AnlagenStatusDeKnoten(final AnlagenStatusModell modell,
			final AnlagenStatusKnoten parent, final SystemObjekt systemObjekt,
			final int level) {
		super(modell, parent, systemObjekt, level);
		setName("DE");
	}

	/** {@inheritDoc} */
	@Override
	public void erzeugeKinder() {
	}

	/** {@inheritDoc} */
	@Override
	public String getLabel() {
		return "DE"
				+ ((De) getSystemObjekt()).getKdDe().getDatum().getDEKanal();
	}

	/** {@inheritDoc} */
	@Override
	AttTlsSysKnotenNummer setKnotenNummer() {
		return new AttTlsSysKnotenNummer(0);
	}

	/** {@inheritDoc} */
	@Override
	public boolean isStatusRelevant() {
		return !isSysDe() && !isCluster() && !isDeKanal255();
	}

	/**
	 * Bestimmt, ob der DE ein System-DE ist (DeSys).
	 * 
	 * @return true, wenn System-DE sonst false
	 */
	private boolean isSysDe() {
		return getSystemObjekt() instanceof DeSys;
	}

	/**
	 * Bestimmt, ob der DE ein Cluster-Kanal ist.
	 * 
	 * @return true, wenn Cluster-Kanal sonst false
	 */
	private boolean isCluster() {
		final Daten datum = ((De) getSystemObjekt()).getKdDe().getDatum();
		return datum.getCluster() == AttJaNein.ZUSTAND_1_JA;
	}

	/**
	 * Bestimmt, ob die Kanalnummer des DE == 255 ist.
	 * 
	 * @return true, wenn Kanalnummer == 255 sonst false
	 */
	private boolean isDeKanal255() {
		final Daten datum = ((De) getSystemObjekt()).getKdDe().getDatum();
		return datum.getDEKanal().intValue() == 255;
	}

	/**
	 * Bearbeitet die empfangenen Daten vom DAV.
	 * 
	 * @param event
	 *            Event
	 */
	private void bearbeiteDavDaten(final DatensatzUpdateEvent event) {
		if (event.getDatum() instanceof OdTlsGloDeFehler.Daten) {
			if (neuerStatus(event.getDatum())) {
				aktualisiereStatus();
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public void aktualisiereStatus() {
		Debug.getLogger()
				.finer(this + "aktualisiereStatus: firePropertyChange");
		firePropertyChange(AnlagenStatusEditPart.STATUS_CHANGED_PROPERTY, null,
				null);
	}

	/**
	 * Bildet einen neuen Status auf der Basis der empfangenen Daten.
	 * 
	 * @param datum
	 *            Daten
	 * @return true, wenn Status ge&auml;ndert, sonst false
	 */
	private boolean neuerStatus(final OnlineDatum datum) {
		final AnlagenStatusKnotenStatus alt = getKnotenStatus();
		if (datum.dContainsDaten()) {
			setKnotenStatus(bildeStatus(datum));
		} else {
			if (!isStatusRelevant()) {
				setKnotenStatus(AnlagenStatusKnotenStatus.UNBEKANNT);
			} else {
				setKnotenStatus(AnlagenStatusKnotenStatus.GESTOERT);
			}
		}

		return alt != getKnotenStatus();
	}

	/**
	 * Bildet den Status auf der Basis der empfangenen Daten.
	 * 
	 * @param datum
	 *            Daten
	 * @return Status
	 */
	private AnlagenStatusKnotenStatus bildeStatus(final OnlineDatum datum) {

		final OdTlsGloDeFehler.Daten d = (OdTlsGloDeFehler.Daten) datum;

		if (d.getDEKanalStatus() == AttTlsDEKanalStatus.ZUSTAND_0_AKTIV
				&& d.getDEFehlerStatus() == AttTlsDEFehlerStatus.ZUSTAND_0_OK) {
			return AnlagenStatusKnotenStatus.OK;
		}

		return AnlagenStatusKnotenStatus.TEILGESTOERT;
	}

	/** {@inheritDoc} */
	@Override
	public void datensatzAktualisiert(final DatensatzUpdateEvent event) {
		Display.getDefault().asyncExec(new Runnable() {

			public void run() {
				bearbeiteDavDaten(event);
			}
		});
	}

	/** {@inheritDoc} */
	@Override
	protected void datenanmelden() {
		if (!(getSystemObjekt() instanceof De)) {
			// System.err.println("KEIN DE-Knoten");
			return;
		}

		final De de = (De) getSystemObjekt();
		de.getOdTlsGloDeFehler().addUpdateListener(
				OdTlsGloDeFehler.Aspekte.TlsAntwort, this);
		anmeldungen.add(de.getOdTlsGloDeFehler());
	}

	/** {@inheritDoc} */
	@Override
	protected void datenabmelden() {
		for (OnlineDatensatz<?> ds : anmeldungen) {
			ds.removeUpdateListener(OdTlsGloDeFehler.Aspekte.TlsAntwort, this);
		}
	}
}
