/*******************************************************************************
 * Copyright (c) 2000, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

package de.bsvrz.buv.plugin.anlagenstatus.modell;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.draw2d.AbstractRouter;
import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.ConnectionRouter;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Ray;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * Provides a {@link Connection} with an orthogonal route between the
 * Connection's source and target anchors.
 * 
 * Erweitert Manhattan-Routing.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: BusRouter.java 33710 2012-04-20 11:06:38Z gieseler $
 */
public final class BusRouter extends AbstractRouter {

	// private final Map rowsUsed = new HashMap();
	private final Map colsUsed = new HashMap();
	// private Hashtable offsets = new Hashtable(7);

	private final Map<Connection, ReservedInfo> reservedInfo = new HashMap<Connection, ReservedInfo>();

	// Gieseler
	private class LineRange {
		private int start;
		private final Connection connection;

		public Connection getConnection() {
			return connection;
		}

		private int end;

		public int getStart() {
			return start;
		}

		public int getEnd() {
			return end;
		}

		public LineRange(final Connection connection, final int rstart,
				final int rend) {
			this.connection = connection;
			if (rstart > rend) {
				start = rend;
				end = rstart;
			} else {
				start = rstart;
				end = rend;
			}
		}
	}

	private final Map<Integer, List<LineRange>> rowMap = new HashMap<Integer, List<LineRange>>();
	private final Map<Integer, List<LineRange>> colMap = new HashMap<Integer, List<LineRange>>();

	// private final Map<ConnectionAnchor, Integer> anchorMap = new
	// HashMap<ConnectionAnchor, Integer>();

	private class ReservedInfo {
		public List reservedRows = new ArrayList(2);
		public List reservedCols = new ArrayList(2);
	}

	private static Ray UP = new Ray(0, -1), DOWN = new Ray(0, 1),
			LEFT = new Ray(-1, 0), RIGHT = new Ray(1, 0);

	/**
	 * @see ConnectionRouter#invalidate(Connection)
	 */
	@Override
	public void invalidate(final Connection connection) {
		removeReservedLines(connection);
	}

	private int getColumnNear(final Connection connection, final int r,
			final int n, final int x, final int startpoint, final int endpoint) {
		int min = Math.min(n, x), max = Math.max(n, x);
		if (min > r) {
			max = min;
			min = r - (min - r);
		}
		if (max < r) {
			min = max;
			max = r + (r - max);
		}

		// Gieseler, Linienbreite beruecksichtigen
		int colinc = 0;
		if (connection instanceof PolylineConnection) {
			final PolylineConnection pc = (PolylineConnection) connection;
			colinc = pc.getLineWidth();
		} else {
			colinc = 1;
		}

		int proximity = 0;
		int direction = -1;
		if (startpoint > endpoint) {
			direction = 1;
		} else {
			direction = -1;
		}

		// Gieseler, unklar was das soll!
		// if (r % 2 == 1)
		// r--;
		Integer i;
		while (proximity < r) {
			i = new Integer(r + proximity * direction);

			if (!isReservedConnectionColumn(connection, new Integer(i),
					startpoint, endpoint)) {

				// Test, ob Endpunkt auf benutzter Zeile liegt
				if (!isReservedConnectionRow(connection, new Integer(endpoint),
						i, i)) {
					reserveConnectionColumn(connection, new Integer(i),
							startpoint, endpoint);
					return i;
				}
			}

			final int j = i.intValue();
			if (j <= min) {
				return j + 2;
			}
			if (j >= max) {
				return j - 2;
			}

			// if (direction == 1)
			// direction = -1;
			// else {
			// direction = 1;
			proximity += 2 * colinc;
			// }
		}
		return r;
	}

	private int getRowNear(final Connection connection, final int row,
			final int n, final int x, final int startpoint, final int endpoint) {
		int min = Math.min(n, x), max = Math.max(n, x);
		if (min > row) {
			max = min;
			min = row - (min - row);
		}
		if (max < row) {
			min = max;
			max = row + (row - max);
		}

		// Gieseler, Linienbreite beruecksichtigen
		int rowinc = 0;
		if (connection instanceof PolylineConnection) {
			final PolylineConnection pc = (PolylineConnection) connection;
			rowinc = pc.getLineWidth();
		} else {
			rowinc = 1;
		}

		int proximity = 0;
		int direction = -1;

		if (startpoint > endpoint) {
			direction = 1;
		} else {
			direction = -1;
		}

		// Gieseler, unklar was das soll!
		// if (r % 2 == 1)
		// r--;
		Integer i;
		while (proximity < row) {
			// Gieseler, nicht abwechselnd drueber und drunter
			i = new Integer(row + proximity * direction);
			// i = new Integer(row + proximity);

			if (!isReservedConnectionRow(connection, new Integer(i),
					startpoint, endpoint)) {
				reserveConnectionRow(connection, new Integer(i), startpoint,
						endpoint);
				return i;
			}

			// if (!rowsUsed.containsKey(i)) {
			// rowsUsed.put(i, i);
			// reserveRow(connection, i);
			// return i.intValue();
			// }

			final int j = i.intValue();
			if (j <= min) {
				return j + rowinc;
			}
			if (j >= max) {
				return j - rowinc;
			}

			// if (direction == 1)
			// direction = -1;
			// else {
			// direction = 1;
			// // proximity += 2;
			//
			// proximity += 2 * row_inc;
			// }
			proximity += 2 * rowinc;
		}
		return row;
	}

	/**
	 * Returns the direction the point <i>p</i> is in relation to the given
	 * rectangle. Possible values are LEFT (-1,0), RIGHT (1,0), UP (0,-1) and
	 * DOWN (0,1).
	 * 
	 * @param r
	 *            the rectangle
	 * @param p
	 *            the point
	 * @return the direction from <i>r</i> to <i>p</i>
	 */
	protected Ray getDirection(final Rectangle r, final Point p) {
		int i;
		int distance = Math.abs(r.x - p.x);
		Ray direction;

		direction = LEFT;

		i = Math.abs(r.y - p.y);
		if (i <= distance) {
			distance = i;
			direction = UP;
		}

		i = Math.abs(r.bottom() - p.y);
		if (i <= distance) {
			distance = i;
			direction = DOWN;
		}

		i = Math.abs(r.right() - p.x);
		if (i < distance) {
			distance = i;
			direction = RIGHT;
		}

		return direction;
	}

	protected Ray getEndDirection(final Connection conn) {
		final ConnectionAnchor anchor = conn.getTargetAnchor();
		final Point p = getEndPoint(conn);
		Rectangle rect;
		if (anchor.getOwner() == null) {
			rect = new Rectangle(p.x - 1, p.y - 1, 2, 2);
		} else {
			rect = conn.getTargetAnchor().getOwner().getBounds().getCopy();
			conn.getTargetAnchor().getOwner().translateToAbsolute(rect);
		}
		return getDirection(rect, p);
	}

	protected Ray getStartDirection(final Connection conn) {
		final ConnectionAnchor anchor = conn.getSourceAnchor();
		final Point p = getStartPoint(conn);
		Rectangle rect;
		if (anchor.getOwner() == null) {
			rect = new Rectangle(p.x - 1, p.y - 1, 2, 2);
		} else {
			rect = conn.getSourceAnchor().getOwner().getBounds().getCopy();
			conn.getSourceAnchor().getOwner().translateToAbsolute(rect);
		}
		return getDirection(rect, p);
	}

	protected void processPositions(final Ray start, final Ray end,
			final List positions, boolean horizontal, final Connection conn) {
		removeReservedLines(conn);

		final int[] pos = new int[positions.size() + 2];
		if (horizontal) {
			pos[0] = start.x;
		} else {
			pos[0] = start.y;
		}

		int i;
		for (i = 0; i < positions.size(); i++) {
			pos[i + 1] = ((Integer) positions.get(i)).intValue();
		}

		if (horizontal == (positions.size() % 2 == 1)) {
			pos[++i] = end.x;
		} else {
			pos[++i] = end.y;
		}

		final PointList points = new PointList();
		points.addPoint(new Point(start.x, start.y));
		Point p;
		int current, prev, min, max;
		boolean adjust;

		Integer connrow = null;

		for (i = 2; i < pos.length - 1; i++) {
			horizontal = !horizontal;
			prev = pos[i - 1];
			current = pos[i];

			adjust = (i != pos.length - 2);

			// int connrow_end = -1;

			if (horizontal) {
				connrow = null;
				if (adjust) {
					min = pos[i - 2];
					max = pos[i + 2];
					// Gieseler,
					// pos[i] = current = getRowNear(conn, current, min, max);

					// neu
					// pos[i] = current;
					connrow = findRow(conn, current);
					if (connrow != -1) {
						pos[i] = current = connrow;
					} else {
						pos[i] = current = getRowNear(conn, current, min, max,
								pos[i - 1], pos[i + 1]);
						connrow = new Integer(current);
						// anchorMap.put(conn.getSourceAnchor(), current);
					}
				}
				p = new Point(prev, current);
			} else {
				if (adjust) {
					min = pos[i - 2];
					max = pos[i + 2];
					final int fcol = findCol(conn, current);
					if (fcol != -1) {
						pos[i] = current = fcol;
					} else {
						pos[i] = current = getColumnNear(conn, current, min,
								max, pos[i - 1], pos[i + 1]);
					}
					reserveConnectionRow(conn, prev, pos[i - 2], current);
				}
				p = new Point(current, prev);
			}
			points.addPoint(p);
		}
		points.addPoint(new Point(end.x, end.y));
		conn.setPoints(points);
	}

	private void reserveConnectionRow(final Connection connection,
			final Integer connrow, final int rowStart, final int rowEnd) {
		final LineRange lr = new LineRange(connection, rowStart, rowEnd);

		List<LineRange> list = rowMap.get(connrow);
		if (list == null) {
			list = new ArrayList<LineRange>();
			list.add(lr);
			rowMap.put(connrow, list);
		} else {
			list.add(lr);
		}

		ReservedInfo info = reservedInfo.get(connection);
		if (info == null) {
			info = new ReservedInfo();
			reservedInfo.put(connection, info);
		}
		info.reservedRows = list;
	}

	private void reserveConnectionColumn(final Connection connection,
			final Integer connrow, final int rowStart, final int rowEnd) {
		final LineRange lr = new LineRange(connection, rowStart, rowEnd);

		List<LineRange> list = colMap.get(connrow);
		if (list == null) {
			list = new ArrayList<LineRange>();
			list.add(lr);
			colMap.put(connrow, list);
		} else {
			list.add(lr);
		}

		ReservedInfo info = reservedInfo.get(connection);
		if (info == null) {
			info = new ReservedInfo();
			reservedInfo.put(connection, info);
		}
		info.reservedCols = list;
	}

	private boolean isReservedConnectionRow(final Connection connection,
			final Integer connrow, final int startpoint, final int endpoint) {
		final List<LineRange> list = rowMap.get(connrow);

		if (list == null) {
			return false;
		}

		// Linienstrke beruecksichtigen
		int teststartpoint = startpoint;
		int testendpoint = endpoint;

		if (endpoint < startpoint) {
			teststartpoint = endpoint;
			testendpoint = startpoint;
		}

		if (connection instanceof PolylineConnection) {
			final PolylineConnection pc = (PolylineConnection) connection;
			testendpoint += pc.getLineWidth();
			teststartpoint -= pc.getLineWidth();
		}

		for (LineRange lr : list) {
			if ((lr.getStart() <= teststartpoint && lr.getEnd() >= teststartpoint)
					|| ((lr.getStart() <= testendpoint && lr.getEnd() >= testendpoint))) {
				return true;
			}
		}

		return false;
	}

	private boolean isReservedConnectionColumn(final Connection connection,
			final Integer connrow, final int startpoint, final int endpoint) {
		final List<LineRange> list = colMap.get(connrow);

		if (list == null) {
			return false;
		}

		// Linienstrke beruecksichtigen
		int teststartpoint = startpoint;
		int testendpoint = endpoint;

		if (endpoint < startpoint) {
			teststartpoint = endpoint;
			testendpoint = startpoint;
		}

		if (connection instanceof PolylineConnection) {
			final PolylineConnection pc = (PolylineConnection) connection;
			testendpoint -= pc.getLineWidth() + 2;
			teststartpoint += pc.getLineWidth() + 2;
		}

		for (LineRange lr : list) {
			if ((lr.getStart() <= teststartpoint && lr.getEnd() >= teststartpoint)
					|| ((lr.getStart() <= testendpoint && lr.getEnd() >= testendpoint))) {
				return true;
			}
		}

		return false;
	}

	private int findRow(final Connection connection, final int row) {
		final Iterator<Entry<Integer, List<LineRange>>> imap = rowMap
				.entrySet().iterator();
		while (imap.hasNext()) {
			final Entry<Integer, List<LineRange>> irow = imap.next();
			final Iterator<LineRange> ilinerange = irow.getValue().iterator();
			while (ilinerange.hasNext()) {
				final LineRange linerange = ilinerange.next();
				if (linerange.getConnection().getSourceAnchor() == connection
						.getSourceAnchor()) {
					return irow.getKey();
				}
			}
		}

		return -1;
	}

	private int findCol(final Connection connection, final int col) {
		final Iterator<Entry<Integer, List<LineRange>>> imap = colMap
				.entrySet().iterator();
		while (imap.hasNext()) {
			final Entry<Integer, List<LineRange>> icolumn = imap.next();
			final Iterator<LineRange> ilinerange = icolumn.getValue()
					.iterator();
			while (ilinerange.hasNext()) {
				final LineRange linerange = ilinerange.next();
				if (linerange.getConnection().getSourceAnchor() == connection
						.getSourceAnchor()) {
					return icolumn.getKey();
				}
			}
		}

		return -1;
	}

	/**
	 * @see ConnectionRouter#remove(Connection)
	 */
	@Override
	public void remove(final Connection connection) {
		removeReservedLines(connection);
	}

	protected void removeReservedLines(final Connection connection) {
		final ReservedInfo rInfo = reservedInfo.get(connection);
		if (rInfo == null) {
			return;
		}

		final Iterator<Entry<Integer, List<LineRange>>> irow = rowMap
				.entrySet().iterator();
		while (irow.hasNext()) {
			final Entry<Integer, List<LineRange>> next = irow.next();
			final Iterator<LineRange> ilinerange = next.getValue().iterator();
			while (ilinerange.hasNext()) {
				final LineRange linerange = ilinerange.next();
				if (linerange.getConnection() == connection) {
					ilinerange.remove();
				}
			}
		}

		final Iterator<Entry<Integer, List<LineRange>>> icolunm = colMap
				.entrySet().iterator();
		while (icolunm.hasNext()) {
			final Entry<Integer, List<LineRange>> next = icolunm.next();
			final Iterator<LineRange> ilinerange = next.getValue().iterator();
			while (ilinerange.hasNext()) {
				final LineRange linerange = ilinerange.next();
				if (linerange.getConnection() == connection) {
					ilinerange.remove();
				}
			}
		}

		// for (int i = 0; i < rInfo.reservedRows.size(); i++) {
		// rowsUsed.remove(rInfo.reservedRows.get(i));
		// }
		for (int i = 0; i < rInfo.reservedCols.size(); i++) {
			colsUsed.remove(rInfo.reservedCols.get(i));
		}
		reservedInfo.remove(connection);
	}

	protected void reserveColumn(final Connection connection,
			final Integer column) {
		ReservedInfo info = reservedInfo.get(connection);
		if (info == null) {
			info = new ReservedInfo();
			reservedInfo.put(connection, info);
		}
		info.reservedCols.add(column);
	}

	protected void reserveRow(final Connection connection, final Integer row) {
		ReservedInfo info = reservedInfo.get(connection);
		if (info == null) {
			info = new ReservedInfo();
			reservedInfo.put(connection, info);
		}
		info.reservedRows.add(row);
	}

	/**
	 * @see ConnectionRouter#route(Connection)
	 */
	public void route(final Connection conn) {
		if ((conn.getSourceAnchor() == null)
				|| (conn.getTargetAnchor() == null)) {
			return;
		}

		int i;
		final Point startPoint = getStartPoint(conn);
		conn.translateToRelative(startPoint);
		final Point endPoint = getEndPoint(conn);
		conn.translateToRelative(endPoint);

		final Ray start = new Ray(startPoint);
		final Ray end = new Ray(endPoint);
		final Ray average = start.getAveraged(end);

		final Ray direction = new Ray(start, end);
		final Ray startNormal = getStartDirection(conn);
		final Ray endNormal = getEndDirection(conn);

		final List positions = new ArrayList(5);
		boolean horizontal = startNormal.isHorizontal();
		if (horizontal) {
			positions.add(new Integer(start.y));
		} else {
			positions.add(new Integer(start.x));
		}

		horizontal = !horizontal;

		if (startNormal.dotProduct(endNormal) == 0) {
			if ((startNormal.dotProduct(direction) >= 0)
					&& (endNormal.dotProduct(direction) <= 0)) {
				// 0
			} else {
				// 2
				if (startNormal.dotProduct(direction) < 0) {
					i = startNormal.similarity(start.getAdded(startNormal
							.getScaled(10)));
				} else {
					if (horizontal) {
						i = average.y;
					} else {
						i = average.x;
					}
				}
				positions.add(new Integer(i));
				horizontal = !horizontal;

				if (endNormal.dotProduct(direction) > 0) {
					i = endNormal.similarity(end.getAdded(endNormal
							.getScaled(10)));
				} else {
					if (horizontal) {
						i = average.y;
					} else {
						i = average.x;
					}
				}
				positions.add(new Integer(i));
				horizontal = !horizontal;
			}
		} else {
			if (startNormal.dotProduct(endNormal) > 0) {
				// 1
				if (startNormal.dotProduct(direction) >= 0) {
					i = startNormal.similarity(start.getAdded(startNormal
							.getScaled(10)));
				} else {
					i = endNormal.similarity(end.getAdded(endNormal
							.getScaled(10)));
				}
				positions.add(new Integer(i));
				horizontal = !horizontal;
			} else {
				// 3 or 1
				if (startNormal.dotProduct(direction) < 0) {
					i = startNormal.similarity(start.getAdded(startNormal
							.getScaled(10)));
					positions.add(new Integer(i));
					horizontal = !horizontal;
				}

				if (horizontal) {
					i = average.y;
				} else {
					i = average.x;
				}
				positions.add(new Integer(i));
				horizontal = !horizontal;

				if (startNormal.dotProduct(direction) < 0) {
					i = endNormal.similarity(end.getAdded(endNormal
							.getScaled(10)));
					positions.add(new Integer(i));
					horizontal = !horizontal;
				}
			}
		}
		if (horizontal) {
			positions.add(new Integer(end.y));
		} else {
			positions.add(new Integer(end.x));
		}

		processPositions(start, end, positions, startNormal.isHorizontal(),
				conn);
	}
}
