/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.ConnectionEndpointLocator;
import org.eclipse.draw2d.ConnectionRouter;
import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;

import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.ColorUtilities;
import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.PluginEinstellungen;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusConnection;

/**
 * Connection-EditPart.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenStatusConnectionEditPart.java 32959 2012-02-29 15:05:11Z gieseler $
 */
public class AnlagenStatusConnectionEditPart extends AbstractConnectionEditPart
		implements PropertyChangeListener {

	/** {@inheritDoc} */
	@Override
	protected void createEditPolicies() {
	}

	/** {@inheritDoc} */
	@Override
	public void activate() {
		super.activate();
		((AnlagenStatusConnection) getModel()).addPropertyChangeListener(this);
	}

	/** {@inheritDoc} */
	@Override
	public void deactivate() {
		((AnlagenStatusConnection) getModel())
				.removePropertyChangeListener(this);
		super.deactivate();
	}

	/** {@inheritDoc} */
	@Override
	public void propertyChange(final PropertyChangeEvent iEvent) {
		// final String property = iEvent.getPropertyName();
	}

	/** {@inheritDoc} */
	@Override
	protected IFigure createFigure() {
		final IFigure connectionFigure = super.createFigure();

		final PolylineConnection p = (PolylineConnection) connectionFigure;
		p.setLineWidth(PluginEinstellungen.getInstance().getEinstellungen()
				.getLinienBreite());

		final AnlagenStatusConnection cm = (AnlagenStatusConnection) getModel();

		connectionFigure.setForegroundColor(ColorUtilities
				.longToColor(PluginEinstellungen.getInstance()
						.getEinstellungen().getLinienFarbe()));

		// ((PolylineConnection) connectionFigure)
		// .setTargetDecoration(new PolygonDecoration());

		final Connection c = ((PolylineConnection) connectionFigure);

		final ConnectionRouter connectionRouter = ((PolylineConnection) connectionFigure)
				.getConnectionRouter();
		((PolylineConnection) connectionFigure).setConnectionRouter(null);
		((PolylineConnection) connectionFigure)
				.setConnectionRouter(connectionRouter);

		// if (cm.isSichtbar()) {
		if (cm.istMitEndeLabel()) {
			final ConnectionEndpointLocator targetEndpointLocator = new ConnectionEndpointLocator(
					c, true);
			targetEndpointLocator.setVDistance(5);
			targetEndpointLocator.setUDistance(5);
			final Label targetMultiplicityLabel = new Label(""
					+ cm.getTargetPort());
			c.add(targetMultiplicityLabel, targetEndpointLocator);
		}
		if (cm.istMitAnfangLabel()) {
			final ConnectionEndpointLocator sourceEndpointLocator = new ConnectionEndpointLocator(
					c, false);
			sourceEndpointLocator.setVDistance(5);
			sourceEndpointLocator.setUDistance(5);
			final Label sourceMultiplicityLabel = new Label(""
					+ cm.getSourcePort());
			c.add(sourceMultiplicityLabel, sourceEndpointLocator);
		}

		connectionFigure.setToolTip(erzeugeTooltip());
		// } else {
		// // p.setLineWidth(0);
		// // // connectionFigure.setForegroundColor(new Color(PlatformUI
		// // // .getWorkbench().getDisplay(), 255, 255, 255));
		// //
		// //
		// connectionFigure.setForegroundColor(ColorConstants.listBackground);
		// connectionFigure.setVisible(false);
		//
		// }

		return connectionFigure;
	}

	/**
	 * Erzeugt den Tooltip f&uuml;r die Verbindung.
	 * 
	 * @return Tooltip-Figure
	 */
	private IFigure erzeugeTooltip() {
		final RectangleFigure rectangle = new RectangleFigure();

		final AnlagenStatusConnection cm = (AnlagenStatusConnection) getModel();

		final FlowLayout flowLayout = new FlowLayout();
		flowLayout.setMajorAlignment(FlowLayout.ALIGN_CENTER);
		flowLayout.setMinorAlignment(FlowLayout.ALIGN_CENTER);
		rectangle.setLayoutManager(flowLayout);

		final Label mLabel = new Label();
		mLabel.setLabelAlignment(PositionConstants.CENTER);
		mLabel.setLabelAlignment(PositionConstants.MIDDLE);

		mLabel.setText(" " + cm.toString() + " ");
		rectangle.add(mLabel);

		return rectangle;
	}

	/** {@inheritDoc} */
	@Override
	protected void refreshVisuals() {
		final AnlagenStatusConnection cm = (AnlagenStatusConnection) getModel();
		if (cm.getTarget().isSichtbar()) {
			super.refreshVisuals();
		} else {
			getConnectionFigure().setVisible(false);
		}

		//
		// final PolylineConnection p = (PolylineConnection)
		// getConnectionFigure();
		// p.setLineWidth(PluginEinstellungen.getInstance().getEinstellungen()
		// .getLinienBreite());
		// p.setForegroundColor(ColorUtilities.longToColor(PluginEinstellungen
		// .getInstance().getEinstellungen().getLinienFarbe()));
	}
}
