/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.anlagenstatus.parts;

import java.util.Collections;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartListener;

import de.bsvrz.buv.plugin.anlagenstatus.export.HtmlTools;
import de.bsvrz.buv.plugin.anlagenstatus.figures.DeFigure;
import de.bsvrz.buv.plugin.anlagenstatus.figures.DeStatusFigure;
import de.bsvrz.buv.plugin.anlagenstatus.figures.StatusFigure;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusConnection;
import de.bsvrz.buv.plugin.anlagenstatus.tooltips.AnlagenStatusToolTipFigure;
import de.bsvrz.buv.plugin.dobj.model.BitCtrlDoModel;
import de.bsvrz.buv.plugin.dobj.model.DobjPackage;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatum.Status;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.De;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.onlinedaten.OdTlsGloDeFehler;

/**
 * DE-Editpart.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * 
 * @version $Id: DeEditPart.java 33260 2012-03-26 14:24:12Z gieseler $
 */
public class DeEditPart extends AnlagenStatusEditPart implements
		EditPartListener {

	/**
	 * ID des EditParts.
	 */
	public static final String PART_ID = DeEditPart.class.getName();

	/** {@inheritDoc} */
	@Override
	protected IFigure createFigure() {
		final IFigure figure = new DeFigure((De) getModel().getSystemObjekt());
		figure.setToolTip(AnlagenStatusToolTipFigure.getTooltipFigure(this));
		addEditPartListener(this);
		return figure;
	}

	/** {@inheritDoc} */
	@Override
	protected void erzeugeFigureInhalt() {
		getFigure().setToolTip(
				AnlagenStatusToolTipFigure.getTooltipFigure(this));
	}

	/** {@inheritDoc} */
	@Override
	public String getHtmlText(final int col, final int cols, final boolean first) {
		final StringBuffer sb = new StringBuffer();
		final SystemObjekt systemobjekt = getModel().getSystemObjekt();

		final De de = (De) systemobjekt;
		final OdTlsGloDeFehler.Daten deDaten = de.getOdTlsGloDeFehler()
				.getDatum(OdTlsGloDeFehler.Aspekte.TlsAntwort);

		sb.append("<table class=\"liste\" border=\"5\" bordercolor=\""
				+ getHtmlBorderColor() + "\" rules=\"none\" >\n");

		sb.append(HtmlTools.erzeugeHtmlTabelleZeile(null,
				HtmlTools.erzeugeHtmlTabelleSpalte(null, getHtmlBasisInfo())));

		sb.append("<tr><td><table border=\"0\" rules=\"none\" >\n");
		String statustext = "";
		if (deDaten == null) {
			statustext = "unbekannt";
		} else {
			if (deDaten.dGetDatenStatus() != Status.DATEN) {
				statustext = deDaten.dGetDatenStatus().toString();
			}
		}

		sb.append(HtmlTools.erzeugeHtmlTabelleZeile(
				null,
				HtmlTools.erzeugeHtmlTabelleSpalte(null, "Status:")
						+ HtmlTools.erzeugeHtmlTabelleSpalte(null, statustext)));

		if (deDaten != null) {
			if (deDaten.dGetDatenStatus() == Status.DATEN) {
				sb.append(HtmlTools.erzeugeHtmlTabelleZeile(
						null,
						HtmlTools.erzeugeHtmlTabelleSpalte(null,
								"TLS-DE-Fehler:")
								+ HtmlTools.erzeugeHtmlTabelleSpalte(null,
										deDaten.getDEFehlerStatus().toString())));
				sb.append(HtmlTools.erzeugeHtmlTabelleZeile(
						null,
						HtmlTools.erzeugeHtmlTabelleSpalte(null,
								"TLS-Kanalstatus:")
								+ HtmlTools.erzeugeHtmlTabelleSpalte(null,
										deDaten.getDEKanalStatus().toString())));
				sb.append(HtmlTools.erzeugeHtmlTabelleZeile(
						null,
						HtmlTools.erzeugeHtmlTabelleSpalte(null,
								"TLS-Projektierungstatus:")
								+ HtmlTools.erzeugeHtmlTabelleSpalte(null,
										deDaten.getDEProjektierungsStatus()
												.toString())));
				sb.append(HtmlTools.erzeugeHtmlTabelleZeile(
						null,
						HtmlTools.erzeugeHtmlTabelleSpalte(null, "Hersteller:")
								+ HtmlTools.erzeugeHtmlTabelleSpalte(null,
										deDaten.getHersteller().toString())));

			}
		}
		sb.append("</table></td></tr>\n");

		sb.append("</table>");

		return sb.toString();
	}

	/** {@inheritDoc} */
	@Override
	public void childAdded(final EditPart child, final int index) {

		erzeugeFigureInhalt();

		if (isInView()) {
			refreshVisuals();
		}
	}

	/** {@inheritDoc} */
	@Override
	public void partActivated(final EditPart editpart) {
	}

	/** {@inheritDoc} */
	@Override
	public void partDeactivated(final EditPart editpart) {
	}

	/** {@inheritDoc} */
	@Override
	public void removingChild(final EditPart child, final int index) {
	}

	/** {@inheritDoc} */
	@Override
	public void selectedStateChanged(final EditPart editpart) {
	}

	/** {@inheritDoc} */
	@Override
	public void notifyChanged(final Notification notification) {
		super.notifyChanged(notification);

		final Object notifier = notification.getNotifier();
		final int type = notification.getEventType();

		if (notifier instanceof BitCtrlDoModel<?>) {
			final int featureID = notification
					.getFeatureID(BitCtrlDoModel.class);
			switch (type) {
			case Notification.SET:
				switch (featureID) {
				case DobjPackage.BIT_CTRL_DO_MODEL__SYSTEM_OBJEKT:
					// childParts.clear();
					refreshChildren();
					break;
				default:
					break;
				}
				break;
			default:
				break;
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	protected StatusFigure erzeugeStatusFigure() {
		return new DeStatusFigure(this);
	}

	/** {@inheritDoc} */
	@Override
	protected List<AnlagenStatusConnection> getModelSourceConnections() {
		return Collections.emptyList();
	}

	/** {@inheritDoc} */
	@Override
	protected List<AnlagenStatusConnection> getModelTargetConnections() {
		return Collections.emptyList();
	}

	/** {@inheritDoc} */
	@Override
	void setPositionUndGroesse() {
	}
}
