/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.parts;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.Assert;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Panel;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartListener;

import de.bsvrz.buv.plugin.anlagenstatus.figures.AnlagenGeraetFigure;
import de.bsvrz.buv.plugin.anlagenstatus.figures.GeraetStatusFigure;
import de.bsvrz.buv.plugin.anlagenstatus.figures.StatusFigure;
import de.bsvrz.buv.plugin.anlagenstatus.figures.SteuerModulFigure;
import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusKnoten;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusModell;
import de.bsvrz.buv.plugin.anlagenstatus.tooltips.AnlagenStatusToolTipFigure;
import de.bsvrz.buv.plugin.dobj.model.BitCtrlDoModel;
import de.bsvrz.buv.plugin.dobj.model.ConfiguratedDoTyp;
import de.bsvrz.buv.plugin.dobj.model.DobjPackage;
import de.bsvrz.buv.plugin.dobj.util.DoTypenUtil;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegendeBaustein;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Geraet;

/**
 * Steuermodul-Editpart.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * 
 * @version $Id: SteuerModulEditPart.java 43864 2013-05-14 11:10:42Z hoesel $
 */
public class SteuerModulEditPart extends AnlagenStatusEditPart implements
		EditPartListener, ILegendeBaustein {

	/**
	 * Kinder.
	 */
	private final List<AnlagenStatusEditPart> childParts = new ArrayList<AnlagenStatusEditPart>();

	/**
	 * Modell-Kinder.
	 */
	private List<BitCtrlDoModel<SystemObjekt>> modellKinder;

	/** {@inheritDoc} */
	@Override
	protected IFigure createFigure() {
		final IFigure figure = new SteuerModulFigure((Geraet) getModel()
				.getSystemObjekt());
		figure.setToolTip(AnlagenStatusToolTipFigure.getTooltipFigure(this));

		addEditPartListener(this);
		return figure;
	}

	/** {@inheritDoc} */
	@Override
	protected void erzeugeFigureInhalt() {
		((AnlagenGeraetFigure) getFigure())
				.updateFigureContent((Geraet) getModel().getSystemObjekt());
		positioniereKinder();
		((SteuerModulFigure) getFigure()).updateFigure();
		((SteuerModulFigure) getFigure()).handleZoomChanged(1.0);
		getFigure().setToolTip(
				AnlagenStatusToolTipFigure.getTooltipFigure(this));
	}

	/**
	 * Positioniert die Kinder.
	 */
	private void positioniereKinder() {
		final Panel dataPanel = ((AnlagenGeraetFigure) getFigure())
				.getDataPanel();
		final Dimension figureSize = dataPanel.getSize();

		final Insets insets = getFigure().getInsets();

		for (AnlagenStatusEditPart part : childParts) {
			final Dimension childSize = part.getFigure().getSize();
			final Point childLocation = new Point(0, figureSize.height);
			figureSize.height += childSize.height;
			figureSize.width = Math.max(figureSize.width, childSize.width);

			getFigure().add(part.getFigure());

			final Rectangle bounds = new Rectangle(childLocation.x,
					childLocation.y, childSize.width, childSize.height);

			getFigure().getLayoutManager().setConstraint(part.getFigure(),
					bounds);
			part.getFigure().setBounds(bounds);
			part.getModel().setLocation(childLocation);
		}

		// figureSize.width += insets.left;
		figureSize.width += insets.left + insets.right;
		figureSize.height += insets.bottom + insets.top;
		getFigure().setSize(figureSize);

		// Kopfbereich zentrieren
		final Dimension size = dataPanel.getSize();
		size.width = figureSize.width;
		dataPanel.setSize(size);

		getFigure().getLayoutManager().setConstraint(dataPanel,
				new Rectangle(0, 0, size.width, size.height));
		((AnlagenGeraetFigure) getFigure()).setDimension(figureSize);
	}

	/** {@inheritDoc} */
	@Override
	public void childAdded(final EditPart child, final int index) {
		if (!childParts.contains(child)) {
			childParts.add((AnlagenStatusEditPart) child);
		}

		erzeugeFigureInhalt();

		refreshVisuals();
	}

	/** {@inheritDoc} */
	@Override
	public void notifyChanged(final Notification notification) {
		super.notifyChanged(notification);

		final Object notifier = notification.getNotifier();
		final int type = notification.getEventType();

		if (notifier instanceof BitCtrlDoModel<?>) {
			final int featureID = notification
					.getFeatureID(BitCtrlDoModel.class);
			switch (type) {
			case Notification.SET:
				switch (featureID) {
				case DobjPackage.BIT_CTRL_DO_MODEL__SYSTEM_OBJEKT:
					modellKinder.clear();
					modellKinder = null;
					childParts.clear();
					refreshChildren();
					break;
				default:
					break;
				}
				break;
			default:
				break;
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public void partActivated(final EditPart editpart) {
	}

	/** {@inheritDoc} */
	@Override
	public void partDeactivated(final EditPart editpart) {
	}

	/** {@inheritDoc} */
	@Override
	public void removingChild(final EditPart child, final int index) {
	}

	/** {@inheritDoc} */
	@Override
	public void selectedStateChanged(final EditPart editpart) {
	}

	/** {@inheritDoc} */
	@Override
	protected List<?> getModelChildren() {
		if (modellKinder != null) {
			return modellKinder;
		}

		final AnlagenStatusModell anlagenModell = PluginAnlagenStatus
				.getDefault().getAnlagenModell();

		final AnlagenStatusKnoten knoten = anlagenModell.getKnoten(getModel()
				.getSystemObjekt());

		modellKinder = new ArrayList<BitCtrlDoModel<SystemObjekt>>();

		if (knoten == null || knoten.getKinder().size() == 0) {
			return modellKinder;
		}

		final ConfiguratedDoTyp doTyp = DoTypenUtil
				.getConfiguratedDoTyp(EakEditPart.PART_ID);

		Assert.isNotNull(doTyp, "Der DO-Typ '" + DeEditPart.PART_ID
				+ "' konnte nicht gefunden werden!");

		// Liste als BitCtrlDoModel<SystemObjekt> zusammenstellen
		for (final AnlagenStatusKnoten k : knoten.getKinder()) {
			// @SuppressWarnings("unchecked")
			// final BitCtrlDoModel<SystemObjekt> bitctrlDoModel =
			// (BitCtrlDoModel<SystemObjekt>) doTyp
			// .createModel();
			// bitctrlDoModel.setSystemObjekt(k.getSystemObjekt());

			k.setDoTyp(doTyp);

			// modellKinder.add(bitctrlDoModel);
			modellKinder.add(k);
		}

		return modellKinder;
	}

	/** {@inheritDoc} */
	@Override
	protected StatusFigure erzeugeStatusFigure() {
		return new GeraetStatusFigure(this);
	}
}
