/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.anlagenstatus.views;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;

import de.bsvrz.buv.rw.basislib.legende.ILegendenAbschnitt;

/**
 * Legende f&umlr Anlagenstatus.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: AnlagenStatusLegende.java 29105 2011-04-07 10:08:11Z gieseler $
 */
public class AnlagenStatusLegende implements ILegendenAbschnitt {

	/**
	 * Legenden-ID.
	 */
	private final String id;

	/**
	 * Legenden-Titel.
	 */
	private final String titel;

	/**
	 * Composite.
	 */
	private AnlagenStatusLegendeComposite legendeComposite;

	/**
	 * Gibt das Composite zur&uuml;ck.
	 * 
	 * @return Composite
	 */
	public AnlagenStatusLegendeComposite getLegendeComposite() {
		return legendeComposite;
	}

	/**
	 * Konstruktor.
	 * 
	 * @param id
	 *            ID
	 * @param titel
	 *            Titel
	 */
	public AnlagenStatusLegende(final String id, final String titel) {
		super();
		this.id = id;
		this.titel = titel;
	}

	/** {@inheritDoc} */
	@Override
	public void erzeugeAbschnittsInhalt(final Composite flaeche) {
		// nur fr Rahmenwerk-Legende
		// flaeche.setLayout(new GridLayout());
		// flaeche.setLayoutData(new GridData(GridData.FILL_BOTH));

		legendeComposite = new AnlagenStatusLegendeComposite(flaeche, SWT.NONE);
		// nur fr Rahmenwerk-Legende
		// flaeche.layout(true);
		// flaeche.getParent().getParent().getParent().layout(true, true);

	}

	/** {@inheritDoc} */
	@Override
	public String getId() {
		return id;
	}

	/** {@inheritDoc} */
	@Override
	public String getTitel() {
		return titel;
	}

	/**
	 * Aktualisiert die Legende.
	 */
	public void aktualisieren() {
		if (legendeComposite != null && !legendeComposite.isDisposed()) {
			legendeComposite.getViewer().refresh();
		}
	}
}
